@@ -4,19 +4,25 @@ import warnings
 
 # submodules
 from .__about__ import __version__  # noqa: F401
+from .utils.localizer import Localizer  # must be imported before others
+
+# instanciate the translator
+localizer = Localizer()
+_ = localizer.tr
 
 # subpackages
-from .parser import Listing, Lookup, Sign  # noqa: F401
-from .converter import (  # noqa: F401
+from .comparator import MetadataComparator  # noqa: E402 F401
+from .parser import Listing, Lookup, Sign  # noqa: E402 F401
+from .converter import (  # noqa: E402 F401
     MATCHER_FORMAT,
     MATCHER_GEOMETRY,
     LookupToIsogeo,
     match_coordinate_system,
 )
-from .parser import MetadataJsonReader, Unzipper  # noqa: F401
+from .parser import MetadataJsonReader, Unzipper  # noqa: E402 F401
 
 # must be imported at last
-from .database.isogeo import MetadataSynchronizer  # noqa: F401
+from .database.isogeo import MetadataSynchronizer  # noqa: E402 F401
 
 # conditional import
 try:
