@@ -56,17 +56,18 @@ def match_coordinate_system(
 
     # match
     if in_epsg_code is not None and in_epsg_code.isdigit():
-        # transform into coordinate-system object
-        out_coordsys = CoordinateSystem(code=in_epsg_code)
-
         # try match between group select SRS and lookup SRS
-        group_srs_epsg = [int(i.get("code")) for i in group_srs]
-        if int(in_epsg_code) in group_srs_epsg:
+        srs_match = [d for d in group_srs if int(d.get("code")) == int(in_epsg_code)]
+        if len(srs_match):
             logger.debug(
                 "EPSG code matched group selected ESPG registry: {}.".format(
                     in_epsg_code
                 )
             )
+            # use Isogeo name instead
+            out_coordsys = CoordinateSystem(
+                code=in_epsg_code, name=srs_match[0].get("name")
+            )
             has_srs = 1
         else:
             logger.debug(
@@ -74,13 +75,15 @@ def match_coordinate_system(
                     in_epsg_code
                 )
             )
+            # use SRS name found into lookup
+            out_coordsys = CoordinateSystem(code=in_epsg_code, name=in_srs_name)
             has_srs = 2
     elif (
         "FME_0" in in_srs_name and in_srs_name == "" and md_lookup.envelope is not None
     ):
         logger.warning(
-            "FME failed to recognize SRS but still calculated an envelope. "
-            "Update your FME configuration."
+            "FME failed to recognize SRS but still calculated an envelope for: {}. "
+            "Update your FME configuration.".format(md_lookup.name)
         )
         has_srs = 3
     else:
