@@ -9,24 +9,15 @@
 # ##################################
 
 # Standard library
-import json
 import logging
-import re
-import warnings
 from concurrent.futures import ThreadPoolExecutor
-from io import BufferedIOBase
-from pathlib import Path
 from time import gmtime, sleep, strftime
-from typing import Union
 
 # 3rd party
-from isogeo_pysdk import Catalog, Event, FeatureAttribute, Isogeo, Metadata, Workgroup
+from isogeo_pysdk import Event, FeatureAttribute, Isogeo, Metadata, Workgroup
+from scan_metadata_processor.comparator import MetadataComparator
 from scan_metadata_processor.converter import LookupToIsogeo
-from scan_metadata_processor.converter.matchers import (
-    match_coordinate_system,
-    MATCHER_FORMAT,
-    MATCHER_GEOMETRY,
-)
+from scan_metadata_processor.converter.matchers import match_coordinate_system
 
 # #############################################################################
 # ########## Globals ###############
@@ -34,9 +25,6 @@ from scan_metadata_processor.converter.matchers import (
 
 logger = logging.getLogger(__name__)
 
-# regex
-reg_digits_parentesis = re.compile(r"^.*?\([^\d]*(\d+)[^\d]*\).*$")
-
 # ##############################################################################
 # ########## Classes ###############
 # ##################################
@@ -60,171 +48,47 @@ class MetadataSynchronizer:
         # report object to fill
         self.report = report
 
-    def read_json_scanned(
-        self, in_json: Union[Path, BufferedIOBase], isogeo_workgroup: str
+    def apply_coordinate_system(
+        self, target_metadata: Metadata, srs_matcher_result: tuple, group: Workgroup
     ):
-        """Initial method to push JSON metadata to Isogeo database.
+        """Add a coordinate-system to a metadata according to the SRS matcher result.
 
-        :param Union[Path, BufferedIOBase] in_json: input JSON
-        :param str isogeo_workgroup: Isogeo group in which to create the metadata
+        :param Metadata target_metadata: metadata to which to add the coordinate-system
+        :param tuple srs_matcher_result: result of the SRS matcher
+        :param Workgroup group: group to which to add the missing coordinate-system
+
+        :raises ValueError: for unexpected srs matcher result
         """
-        # warn about deprecated method
-        warn_msg = (
-            "This method was the initial tool to push JSON metadata"
-            " to Isogeo. Prefer using ZIP instead."
-        )
-        warnings.warn(warn_msg, DeprecationWarning)
-        logger.warning(warn_msg)
-
-        # process
-        if isinstance(in_json, Path):
-            with in_json.open(mode="r") as f:
-                json_read = json.load(f)
-        if isinstance(in_json, BufferedIOBase):
-            json_read = json.load(in_json)
-
-        dataset_scanned = json_read[0].get("dataset")
-
-        logging.info(
-            "Workgroup '{}' uploaded a metadata about the dataset: {}".format(
-                isogeo_workgroup.name, dataset_scanned.get("name")
+        if srs_matcher_result[0] == 0:
+            logger.warning(
+                "No coordinate-system identified for: {}".format(
+                    target_metadata.title_or_name(slugged=True)
+                )
             )
-        )
-
-        # load main attributes into a metadata object
-        new_metadata = Metadata(
-            name=dataset_scanned.get("name"),
-            title=dataset_scanned.get("name"),
-            series=False,
-            editionProfile="manual",
-            path=str(dataset_scanned.get("path")),
-            features=dataset_scanned.get("numberOfFeatures"),
-            geometry=MATCHER_GEOMETRY.get(dataset_scanned.get("type")),
-            envelope=json.loads(dataset_scanned.get("envelope", [])),
-            language=isogeo_workgroup.metadataLanguage,
-        )
-
-        # get type from matching format
-        md_format = MATCHER_FORMAT.get(dataset_scanned.get("formatShort"))
-        if md_format in self.isogeo_formats_codes:
-            logging.info(
-                "Format '{}' matched Isogeo formats registry: {}".format(
-                    dataset_scanned.get("formatShort"), md_format
+            self.report.li_srs_not_identified.append(target_metadata._id)
+        elif srs_matcher_result[0] == 1:
+            self.isogeo.srs.associate_metadata(
+                metadata=target_metadata, coordinate_system=srs_matcher_result[1]
+            )
+        elif srs_matcher_result[0] == 2:
+            self.isogeo.srs.associate_workgroup(
+                coordinate_system=srs_matcher_result[1], workgroup=group
+            )
+            self.isogeo.srs.associate_metadata(
+                metadata=target_metadata, coordinate_system=srs_matcher_result[1]
+            )
+        elif srs_matcher_result[0] == 3:
+            logger.warning(
+                "Coordinate-system not recognized by FME for: {}".format(
+                    target_metadata.title_or_name(slugged=True)
                 )
             )
-            new_metadata.format = md_format
-
-            # deduce type from format
-            isogeo_format = self.isogeo.formats.get(format_code=md_format)
-
-            # if dataset, be more specific
-            if isogeo_format.type == "dataset" and MATCHER_GEOMETRY.get(
-                dataset_scanned.get("type")
-            ):
-                new_metadata.type = "vectorDataset"
-            else:
-                new_metadata.type = isogeo_format.type
-
         else:
-            new_metadata.type = "resource"
-
-        # create the metadata
-        logging.info("Creating a new {} metadata...".format(new_metadata.type))
-        metadata_created = self.isogeo.metadata.create(
-            workgroup_id=isogeo_workgroup._id, metadata=new_metadata
-        )
-        if not isinstance(metadata_created, Metadata):
-            logging.error("Creation of metadata on Isogeo failed...")
-            return new_metadata
-        else:
-            logging.info("Metadata created on Isogeo: {}".format(metadata_created._id))
-
-        # apply coordinate-system
-        srs_epsg_code = dataset_scanned.get("coordsys", {}).get("EPSG")
-        if srs_epsg_code is not None and srs_epsg_code.isdigit():
-            isogeo_srs = self.isogeo.coordinate_system.get(
-                coordinate_system_code=srs_epsg_code
-            )
-            logging.info(
-                "EPSG code {} matched Isogeo ESPG registry: {}. Applying it to the created metadata...".format(
-                    srs_epsg_code, isogeo_srs.name
+            raise ValueError(
+                "Unexpected value returned by coordinate-system matcher: {}".format(
+                    srs_matcher_result
                 )
             )
-            self.isogeo.coordinate_system.associate_metadata(
-                metadata=metadata_created, coordinate_system=isogeo_srs
-            )
-
-        # apply feature attributes
-        if metadata_created.type == "vectorDataset" and len(
-            dataset_scanned.get("attributes", [])
-        ):
-            logging.info(
-                "{} feature attributes spotted. Let's add them...".format(
-                    len(dataset_scanned.get("attributes"))
-                )
-            )
-            for featattr in dataset_scanned.get("attributes"):
-                # extract length from feature-attribute type
-                length_filter = reg_digits_parentesis.match(featattr.get("type"))
-                if length_filter:
-                    attr_length = length_filter.group(1)
-                    attr_type = featattr.get("type").replace(
-                        "({})".format(attr_length), ""
-                    )
-                else:
-                    attr_length = None
-                    attr_type = featattr.get("type")
-
-                # load in object
-                new_featattr = FeatureAttribute(
-                    name=featattr.get("name"), dataType=attr_type, length=attr_length
-                )
-                # apply insee_com
-                if featattr.get("name").lower() == "insee_com":
-                    new_featattr.alias = "Numéro INSEE de la commune"
-                    new_featattr.description = (
-                        "Il  s’agit  de  la  valeur  de  l’attribut"
-                        " INSEE_COM  de  la  commune  à  laquelle  se  rapporte  le  chef-lieu.\n\n"
-                        "Il permet d’établir un lien entre le ponctuel de la classe CHEF_LIEU "
-                        "et l’objet surfacique de la classe COMMUNE."
-                    )
-
-                #
-                self.isogeo.metadata.attributes.create(
-                    metadata=metadata_created, attribute=new_featattr,
-                )
-
-        # apply catalogs with $scan option
-        wg_catalogs = self.isogeo.catalog.listing(
-            workgroup_id=isogeo_workgroup._id, include=None
-        )
-        for cat in wg_catalogs:
-            if cat.get("$scan", False):
-                self.isogeo.catalog.associate_metadata(
-                    metadata=metadata_created, catalog=Catalog.clean_attributes(cat)
-                )
-                logging.info(
-                    "Metadata {} associated to the catalog: {}".format(
-                        metadata_created._id, cat.get("name")
-                    )
-                )
-
-        # creation event
-        md_evt_creation = Event(
-            date=strftime("%Y-%m-%d", gmtime()),
-            description="Création de la métadonnée par le Scan Offline",
-            kind="update",
-            parent_resource=metadata_created._id,
-        )
-        self.isogeo.metadata.events.create(
-            metadata=metadata_created, event=md_evt_creation
-        )
-
-        # return final metadata
-        return self.isogeo.metadata.get(
-            metadata_created._id,
-            include=("coordinate-system", "feature-attributes", "tags"),
-        )
 
     def create_from_lookup(
         self,
@@ -315,7 +179,7 @@ class MetadataSynchronizer:
         # feature attributes
         try:
             with ThreadPoolExecutor(
-                thread_name_prefix="MdFeatureAttributes"
+                thread_name_prefix="AddFeatureAttributes"
             ) as executor:
                 for featattr in converter.prepare_feature_attributes():
                     executor.submit(
@@ -353,45 +217,90 @@ class MetadataSynchronizer:
         # return signature
         return md_created
 
-    def apply_coordinate_system(
-        self, target_metadata: Metadata, srs_matcher_result: tuple, group: Workgroup
-    ):
-        """Add a coordinate-system to a metadata according to the SRS matcher result.
+    def update_from_comparator(self, comparator: MetadataComparator) -> Metadata:
 
-        :param Metadata target_metadata: metadata to which to add the coordinate-system
-        :param tuple srs_matcher_result: result of the SRS matcher
-        :param Workgroup group: group to which to add the missing coordinate-system
+        """Create a metadata from a Scan lookup model.
 
-        :raises ValueError: for unexpected srs matcher result
+        :param MetadataComparator comparator: comparator object used to perform \
+        deletion/addition operations
+
+        :return: created metadata
+        :rtype: Metadata
         """
-        if srs_matcher_result[0] == 0:
-            logger.warning(
-                "No coordinate-system identified for: {}".format(
-                    target_metadata.title_or_name(slugged=True)
+        # extract objects from comparator
+        md_to_update = comparator.md_old
+        md_from_lookup = comparator.md_new
+        evt_diff = Event(
+            date=strftime("%Y-%m-%d", gmtime()),
+            description=comparator.diff_message(),
+            kind="update",
+        )
+
+        # update features count
+        if comparator.diff_features_count():
+            logger.debug("Updating features count for: {}".format(md_to_update._id))
+            md_to_update.features = md_from_lookup.features
+            self.isogeo.metadata.update(metadata=md_to_update)
+
+        # update coordinate-system
+        if comparator.diff_srs():
+            logger.debug("Changing coordinate-system for: {}".format(md_to_update._id))
+            self.isogeo.coordinate_system.associate_metadata(
+                metadata=md_to_update, coordinate_system=md_from_lookup.coordinateSystem
+            )
+
+        # update feature attributes
+        diff_attrs = comparator.diff_features_attributes()
+        if diff_attrs[0]:
+            # add new attributes
+            try:
+                with ThreadPoolExecutor(
+                    thread_name_prefix="AddFeatureAttributes"
+                ) as executor:
+                    for new_attr in diff_attrs[1]:
+                        executor.submit(
+                            self.isogeo.metadata.attributes.create,
+                            metadata=md_to_update,
+                            attribute=FeatureAttribute(
+                                name=new_attr[0], dataType=new_attr[1]
+                            ),
+                        )
+            except Exception as err:
+                err_msg = "Failed to add feature attributes to metadata: {} ({}). Trace: {}".format(
+                    md_to_update.title_or_name(slugged=True), md_to_update._id, err
                 )
-            )
-            self.report.li_srs_not_identified.append(target_metadata._id)
-        elif srs_matcher_result[0] == 1:
-            self.isogeo.srs.associate_metadata(
-                metadata=target_metadata, coordinate_system=srs_matcher_result[1]
-            )
-        elif srs_matcher_result[0] == 2:
-            self.isogeo.srs.associate_workgroup(
-                coordinate_system=srs_matcher_result[1], workgroup=group
-            )
-            self.isogeo.srs.associate_metadata(
-                metadata=target_metadata, coordinate_system=srs_matcher_result[1]
-            )
-        elif srs_matcher_result[0] == 3:
-            logger.warning(
-                "Coordinate-system not recognized by FME for: {}".format(
-                    target_metadata.title_or_name(slugged=True)
+                logger.error(err_msg)
+
+            # remove deleted attributes
+            # we need the attribute _id, so make a match
+            match_attr_to_remove = [
+                attr
+                for attr in md_to_update.featureAttributes
+                if attr.name in diff_attrs[2]
+            ]
+            try:
+                with ThreadPoolExecutor(
+                    thread_name_prefix="RemoveFeatureAttributes"
+                ) as executor:
+                    for attr in match_attr_to_remove:
+                        executor.submit(
+                            self.isogeo.metadata.attributes.delete,
+                            attribute=attr,
+                            metadata=md_to_update,
+                        )
+            except Exception as err:
+                err_msg = "Failed to add feature attributes to metadata: {} ({}). Trace: {}".format(
+                    md_to_update.title_or_name(slugged=True), md_to_update._id, err
                 )
-            )
-        else:
-            raise ValueError(
-                "Unexpected value returned by coordinate-system matcher: {}".format(
-                    srs_matcher_result
+                logger.error(err_msg)
+
+        # add update event
+        try:
+            self.isogeo.metadata.events.create(metadata=md_to_update, event=evt_diff)
+        except Exception as err:
+            logger.error(
+                "Failed to add update event to metadata: {}. Trace: {}".format(
+                    md_to_update.admin_url(), err
                 )
             )
 
